@@ -16,16 +16,17 @@
 #    under the License.
 
 import copy
-import json
+from http import client as http_client
+import io
 import logging
 import os
 import socket
 import ssl
+from urllib import parse as urlparse
 
 from keystoneauth1 import adapter
+from oslo_serialization import jsonutils
 from oslo_utils import importutils
-import six
-import six.moves.urllib.parse as urlparse
 
 from magnumclient import exceptions
 
@@ -39,32 +40,45 @@ API_VERSION = '/v1'
 DEFAULT_API_VERSION = 'latest'
 
 
-def _extract_error_json(body):
-    """Return error_message from the HTTP response body."""
+def _extract_error_json_text(body_json):
     error_json = {}
-    try:
-        body_json = json.loads(body)
-        if 'error_message' in body_json:
-            raw_msg = body_json['error_message']
-            error_json = json.loads(raw_msg)
-        elif 'error' in body_json:
-            error_body = body_json['error']
-            error_json = {'faultstring': error_body['title'],
-                          'debuginfo': error_body['message']}
-        else:
-            error_body = body_json['errors'][0]
-            error_json = {'faultstring': error_body['title']}
-            if 'detail' in error_body:
-                error_json['debuginfo'] = error_body['detail']
-            elif 'description' in error_body:
-                error_json['debuginfo'] = error_body['description']
-
-    except ValueError:
-        return {}
-
+    if 'error_message' in body_json:
+        raw_msg = body_json['error_message']
+        error_json = jsonutils.loads(raw_msg)
+    elif 'error' in body_json:
+        error_body = body_json['error']
+        error_json = {'faultstring': error_body['title'],
+                      'debuginfo': error_body['message']}
+    else:
+        error_body = body_json['errors'][0]
+        error_json = {'faultstring': error_body['title']}
+        if 'detail' in error_body:
+            error_json['debuginfo'] = error_body['detail']
+        elif 'description' in error_body:
+            error_json['debuginfo'] = error_body['description']
     return error_json
 
 
+def _extract_error_json(body, resp):
+    """Return error_message from the HTTP response body."""
+    try:
+        content_type = resp.headers.get("Content-Type", "")
+    except AttributeError:
+        content_type = ""
+    if content_type.startswith("application/json"):
+        try:
+            body_json = resp.json()
+            return _extract_error_json_text(body_json)
+        except ValueError:
+            return {}
+    else:
+        try:
+            body_json = jsonutils.loads(body)
+            return _extract_error_json_text(body_json)
+        except ValueError:
+            return {}
+
+
 class HTTPClient(object):
 
     def __init__(self, endpoint, api_version=DEFAULT_API_VERSION, **kwargs):
@@ -93,7 +107,7 @@ class HTTPClient(object):
             _kwargs['key_file'] = kwargs.get('key_file', None)
             _kwargs['insecure'] = kwargs.get('insecure', False)
         elif parts.scheme == 'http':
-            _class = six.moves.http_client.HTTPConnection
+            _class = http_client.HTTPConnection
         else:
             msg = 'Unsupported scheme: %s' % parts.scheme
             raise exceptions.EndpointException(msg)
@@ -194,13 +208,13 @@ class HTTPClient(object):
             ]
             body_str = ''.join(body_list)
             self.log_http_response(resp, body_str)
-            body_iter = six.StringIO(body_str)
+            body_iter = io.StringIO(body_str)
         else:
             self.log_http_response(resp)
 
         if 400 <= resp.status < 600:
             LOG.warning("Request returned failure status.")
-            error_json = _extract_error_json(body_str)
+            error_json = _extract_error_json(body_str, resp)
             raise exceptions.from_response(
                 resp, error_json.get('faultstring'),
                 error_json.get('debuginfo'), method, url)
@@ -218,7 +232,7 @@ class HTTPClient(object):
         kwargs['headers'].setdefault('Accept', 'application/json')
 
         if 'body' in kwargs:
-            kwargs['body'] = json.dumps(kwargs['body'])
+            kwargs['body'] = jsonutils.dumps(kwargs['body'])
 
         resp, body_iter = self._http_request(url, method, **kwargs)
         content_type = resp.getheader('content-type', None)
@@ -229,7 +243,7 @@ class HTTPClient(object):
         if 'application/json' in content_type:
             body = ''.join([chunk for chunk in body_iter])
             try:
-                body = json.loads(body)
+                body = jsonutils.loads(body)
             except ValueError:
                 LOG.error('Could not decode response body as JSON')
         else:
@@ -244,7 +258,7 @@ class HTTPClient(object):
         return self._http_request(url, method, **kwargs)
 
 
-class VerifiedHTTPSConnection(six.moves.http_client.HTTPSConnection):
+class VerifiedHTTPSConnection(http_client.HTTPSConnection):
     """httplib-compatibile connection using client-side SSL authentication
 
     :see http://code.activestate.com/recipes/
@@ -253,9 +267,9 @@ class VerifiedHTTPSConnection(six.moves.http_client.HTTPSConnection):
 
     def __init__(self, host, port, key_file=None, cert_file=None,
                  ca_file=None, timeout=None, insecure=False):
-        six.moves.http_client.HTTPSConnection.__init__(self, host, port,
-                                                       key_file=key_file,
-                                                       cert_file=cert_file)
+        http_client.HTTPSConnection.__init__(self, host, port,
+                                             key_file=key_file,
+                                             cert_file=cert_file)
         self.key_file = key_file
         self.cert_file = cert_file
         if ca_file is not None:
@@ -346,7 +360,7 @@ class SessionClient(adapter.LegacyJsonAdapter):
                                     raise_exc=False, **kwargs)
 
         if 400 <= resp.status_code < 600:
-            error_json = _extract_error_json(resp.content)
+            error_json = _extract_error_json(resp.content, resp)
             raise exceptions.from_response(
                 resp, error_json.get('faultstring'),
                 error_json.get('debuginfo'), method, url)
@@ -363,7 +377,7 @@ class SessionClient(adapter.LegacyJsonAdapter):
         kwargs['headers'].setdefault('Content-Type', 'application/json')
         kwargs['headers'].setdefault('Accept', 'application/json')
         if 'body' in kwargs:
-            kwargs['data'] = json.dumps(kwargs.pop('body'))
+            kwargs['data'] = jsonutils.dumps(kwargs.pop('body'))
 
         resp = self._http_request(url, method, **kwargs)
         body = resp.content
@@ -385,7 +399,22 @@ class SessionClient(adapter.LegacyJsonAdapter):
         kwargs.setdefault('headers', {})
         kwargs['headers'].setdefault('Content-Type',
                                      'application/octet-stream')
-        return self._http_request(url, method, **kwargs)
+        resp = self._http_request(url, method, **kwargs)
+        body = resp.content
+        status = resp.status_code
+        content_type = resp.headers.get('content-type', None)
+
+        if status == 204 or status == 205 or content_type is None:
+            return resp, list()
+        if 'application/json' in content_type:
+            try:
+                body = resp.json()
+            except ValueError:
+                LOG.error('Could not decode response body as JSON')
+        else:
+            body = None
+
+        return resp, body
 
 
 class ResponseBodyIterator(object):
@@ -396,7 +425,10 @@ class ResponseBodyIterator(object):
 
     def __iter__(self):
         while True:
-            yield self.next()
+            try:
+                yield self.next()
+            except StopIteration:
+                return
 
     def __bool__(self):
         return hasattr(self, 'items')
@@ -408,7 +440,7 @@ class ResponseBodyIterator(object):
         if chunk:
             return chunk
         else:
-            raise StopIteration()
+            raise StopIteration
 
 
 def _construct_http_client(*args, **kwargs):
