@@ -34,10 +34,17 @@ CLUSTER_ATTRIBUTES = [
     'updated_at',
     'coe_version',
     'labels',
+    'labels_overridden',
+    'labels_skipped',
+    'labels_added',
+    'fixed_network',
+    'fixed_subnet',
+    'floating_ip_enabled',
     'faults',
     'keypair',
     'api_address',
     'master_addresses',
+    'master_lb_enabled',
     'create_timeout',
     'node_count',
     'discovery_url',
@@ -47,7 +54,7 @@ CLUSTER_ATTRIBUTES = [
     'master_flavor_id',
     'flavor_id',
     'health_status_reason',
-    'project_id'
+    'project_id',
 ]
 
 
@@ -141,6 +148,20 @@ class CreateCluster(command.Command):
             action='append_const',
             const=False,
             help=_('Disables floating ip creation on the new Cluster'))
+        parser.add_argument(
+            '--merge-labels',
+            dest='merge_labels',
+            action='store_true',
+            default=False,
+            help=_('The labels provided will be merged with the labels '
+                   'configured in the specified cluster template.'))
+        parser.add_argument(
+            '--master-lb-enabled',
+            dest='master_lb_enabled',
+            action='store_true',
+            default=False,
+            help=_('Indicates whether created clusters should have '
+                   'a loadbalancer for API.'))
 
         return parser
 
@@ -185,6 +206,14 @@ class CreateCluster(command.Command):
         if parsed_args.fixed_subnet is not None:
             args["fixed_subnet"] = parsed_args.fixed_subnet
 
+        if parsed_args.merge_labels:
+            # We are only sending this if it's True. This
+            # way we avoid breaking older APIs.
+            args["merge_labels"] = parsed_args.merge_labels
+
+        if parsed_args.master_lb_enabled:
+            args["master_lb_enabled"] = parsed_args.master_lb_enabled
+
         cluster = mag_client.clusters.create(**args)
         print("Request to create cluster %s accepted"
               % cluster.uuid)
@@ -382,10 +411,10 @@ class ConfigCluster(command.Command):
 
         parsed_args.dir = os.path.abspath(parsed_args.dir)
         cluster = mag_client.clusters.get(parsed_args.cluster)
-        if cluster.status not in ('CREATE_COMPLETE', 'UPDATE_COMPLETE',
-                                  'ROLLBACK_COMPLETE'):
-            raise exceptions.CommandError("cluster in status %s" %
-                                          cluster.status)
+        if cluster.api_address is None:
+            self.log.warning("WARNING: The cluster's api_address is"
+                             " not known yet.")
+
         cluster_template = mag_client.cluster_templates.get(
             cluster.cluster_template_id)
         opts = {
