@@ -13,10 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import json
+from http import client as http_client
+import io
+from unittest import mock
 
-import mock
-import six
+from oslo_serialization import jsonutils
 import socket
 
 from magnumclient.common.apiclient.exceptions import GatewayTimeout
@@ -37,7 +38,7 @@ def _get_error_body(faultstring=None, debuginfo=None, err_type=NORMAL_ERROR):
             'faultstring': faultstring,
             'debuginfo': debuginfo
         }
-        raw_error_body = json.dumps(error_body)
+        raw_error_body = jsonutils.dumps(error_body)
         body = {'error_message': raw_error_body}
     elif err_type == ERROR_DICT:
         body = {'error': {'title': faultstring, 'message': debuginfo}}
@@ -47,11 +48,11 @@ def _get_error_body(faultstring=None, debuginfo=None, err_type=NORMAL_ERROR):
     elif err_type == ERROR_LIST_WITH_DESC:
         main_body = {'title': faultstring, 'description': debuginfo}
         body = {'errors': [main_body]}
-    raw_body = json.dumps(body)
+    raw_body = jsonutils.dumps(body)
     return raw_body
 
 
-HTTP_CLASS = six.moves.http_client.HTTPConnection
+HTTP_CLASS = http_client.HTTPConnection
 HTTPS_CLASS = http.VerifiedHTTPSConnection
 DEFAULT_TIMEOUT = 600
 
@@ -81,7 +82,7 @@ class HttpClientTest(utils.BaseTestCase):
     def test_server_exception_empty_body(self):
         error_body = _get_error_body()
         fake_resp = utils.FakeResponse({'content-type': 'application/json'},
-                                       six.StringIO(error_body),
+                                       io.StringIO(error_body),
                                        version=1,
                                        status=500)
         client = http.HTTPClient('http://localhost/')
@@ -97,7 +98,7 @@ class HttpClientTest(utils.BaseTestCase):
         error_msg = 'test error msg'
         error_body = _get_error_body(error_msg, err_type=ERROR_DICT)
         fake_resp = utils.FakeResponse({'content-type': 'application/json'},
-                                       six.StringIO(error_body),
+                                       io.StringIO(error_body),
                                        version=1,
                                        status=500)
         client = http.HTTPClient('http://localhost/')
@@ -116,7 +117,7 @@ class HttpClientTest(utils.BaseTestCase):
         error_body = _get_error_body(error_msg, error_trace,
                                      ERROR_LIST_WITH_DESC)
         fake_resp = utils.FakeResponse({'content-type': 'application/json'},
-                                       six.StringIO(error_body),
+                                       io.StringIO(error_body),
                                        version=1,
                                        status=500)
         client = http.HTTPClient('http://localhost/')
@@ -270,7 +271,7 @@ class HttpClientTest(utils.BaseTestCase):
     def test_401_unauthorized_exception(self):
         error_body = _get_error_body(err_type=ERROR_LIST_WITH_DETAIL)
         fake_resp = utils.FakeResponse({'content-type': 'text/plain'},
-                                       six.StringIO(error_body),
+                                       io.StringIO(error_body),
                                        version=1,
                                        status=401)
         client = http.HTTPClient('http://localhost/')
@@ -299,7 +300,7 @@ class HttpClientTest(utils.BaseTestCase):
         err = "foo"
         fake_resp = utils.FakeResponse(
             {'content-type': 'application/json'},
-            six.StringIO(err), version=1, status=200)
+            io.StringIO(err), version=1, status=200)
         client = http.HTTPClient('http://localhost/')
         conn = utils.FakeConnection(fake_resp)
         client.get_connection = (lambda *a, **kw: conn)
@@ -325,7 +326,7 @@ class HttpClientTest(utils.BaseTestCase):
     def test_server_success_body_none(self):
         fake_resp = utils.FakeResponse(
             {'content-type': None},
-            six.StringIO('bar'), version=1, status=200)
+            io.StringIO('bar'), version=1, status=200)
         client = http.HTTPClient('http://localhost/')
         conn = utils.FakeConnection(fake_resp)
         client.get_connection = (lambda *a, **kw: conn)
@@ -339,7 +340,7 @@ class HttpClientTest(utils.BaseTestCase):
         err = _get_error_body()
         fake_resp = utils.FakeResponse(
             {'content-type': 'application/json'},
-            six.StringIO(err), version=1, status=200)
+            io.StringIO(err), version=1, status=200)
         client = http.HTTPClient('http://localhost/')
         conn = utils.FakeConnection(fake_resp)
         client.get_connection = (lambda *a, **kw: conn)
@@ -347,7 +348,7 @@ class HttpClientTest(utils.BaseTestCase):
         resp, body = client.json_request('GET', '/v1/resources')
 
         self.assertEqual(resp, fake_resp)
-        self.assertEqual(json.dumps(body), err)
+        self.assertEqual(jsonutils.dumps(body), err)
 
     def test_raw_request(self):
         fake_resp = utils.FakeResponse(
@@ -449,10 +450,11 @@ class SessionClientTest(utils.BaseTestCase):
         client = http.SessionClient(
             session=fake_session, endpoint_override='http://magnum')
 
-        resp = client.raw_request('GET', '/v1/bays')
+        resp, resp_body = client.raw_request('GET', '/v1/bays')
 
         self.assertEqual(
             fake_session.request.call_args[1]['headers']['Content-Type'],
             'application/octet-stream'
         )
+        self.assertEqual(None, resp_body)
         self.assertEqual(fake_response, resp)
